@@ -40,6 +40,7 @@ class CliProgress(ProgressInterface):
         self.t0 = time.time()
         self.t = self.t0
         self.line_length = 0
+        self._update_precision = decimals + 2
         if initial_print:
             self.print_current()
 
@@ -47,10 +48,12 @@ class CliProgress(ProgressInterface):
         self.prefix = label
 
     def update(self, completed_fraction: float) -> None:
-        self.n = completed_fraction
         t = time.time()
-        if round(completed_fraction, self.decimals) > round(self.n, self.decimals) \
-           or t - self.t > 1.:
+        completed_fraction_rounded = round(completed_fraction,
+                                           self._update_precision)
+        # != also handles reseting progress
+        if completed_fraction_rounded != self.n or t - self.t > 1.:
+            self.n = completed_fraction_rounded
             self.t = t
             self.print_current()
 
