@@ -1,18 +1,17 @@
-import io
-import os
 import enum
+import io
 import json
+import os
 import re
 import urllib
-from pathlib import Path
-from typing import List, Optional, Tuple, Union, Type, IO, cast
 import warnings
+from pathlib import Path
+from typing import List, Optional, Tuple, Union, Type, IO, Callable, cast
 from functools import wraps, partial
 
 import gpg_lite as gpg
 from libbiomedit import crypt
 from .error import UserError
-from .filesystem import delete_files
 from .request import urlopen
 
 GPGStore = gpg.GPGStore
@@ -270,21 +269,22 @@ def assert_key_is_not_revoked(key: gpg.Key):
         raise UserError(f"{key.uids[0]} key has been revoked")
 
 
-def encrypt_and_sign_file(file_object_to_encrypt: IO[bytes],
-                          output_file: str,
-                          gpg_store: GPGStore,
-                          recipients_fingerprint: List[str],
-                          signature_fingerprint: Optional[str] = None,
-                          passphrase: Optional[str] = None,
-                          always_trust: bool = True) -> None:
-    """Encrypt input file with a GnuPG public key and sign it with a GnuPG
+def encrypt_and_sign(source: Callable,
+                     output: Callable,
+                     gpg_store: GPGStore,
+                     recipients_fingerprint: List[str],
+                     signature_fingerprint: Optional[str] = None,
+                     passphrase: Optional[str] = None,
+                     always_trust: bool = True):
+    """Encrypt input data with a GnuPG public key and sign it with a GnuPG
     private key.
+
     Note: in this function we do not check the validity of the sender and
     recipient keys, because it is assumed that this is already done earlier
     in the workflow.
 
-    :param file_object_to_encrypt: file object of the file to be encrypted.
-    :param output_file: path of output encrypted file.
+    :param source: callable writing data to encrypt.
+    :param output: callable reading encrypted data.
     :param gpg_store: directory containing GnuPG keyrings as gnupg object.
     :param recipients_fingerprint: fingerprint of public key(s) with which
         the data should be encrypted.
@@ -298,22 +298,18 @@ def encrypt_and_sign_file(file_object_to_encrypt: IO[bytes],
         local user.
     :raises UserError:
     """
-    with file_object_to_encrypt as f, open(output_file, 'wb') as fout:
-        try:
-            gpg_store.encrypt(
-                source=f,
-                recipients=recipients_fingerprint,
-                sign=signature_fingerprint,
-                output=fout,
-                passphrase=passphrase,
-                trust_model=gpg.TrustModel.always if always_trust
-                else gpg.TrustModel.pgp)
-        except gpg.cmd.GPGError:
-            delete_files(output_file)
-            raise UserError(f'encryption failed for file [{output_file}]. '
-                            f'Maybe you entered a wrong passphrase.') from None
-    if not Path(output_file).is_file():
-        raise UserError(f'encryption failed for [{output_file}].')
+    try:
+        gpg_store.encrypt(
+            source=source,
+            recipients=recipients_fingerprint,
+            sign=signature_fingerprint,
+            output=output,
+            passphrase=passphrase,
+            trust_model=gpg.TrustModel.always if always_trust
+            else gpg.TrustModel.pgp)
+    except gpg.cmd.GPGError:
+        raise UserError("Encryption failed. "
+                        "Maybe you entered a wrong passphrase.") from None
 
 
 def get_recipient_email(key: gpg.Key) -> str:
@@ -325,21 +321,36 @@ def get_recipient_email(key: gpg.Key) -> str:
     return email
 
 
-def decrypt(f: io.FileIO, dest: str, gpg_store: GPGStore,
-            passphrase: Optional[str]):
-    """Decrypt data"""
-    with f as stream, open(dest, "wb") as fout:
-        try:
-            with gpg_store.decrypt(
-                    source=stream,
+def decrypt(source: Union[io.FileIO, Callable],
+            output: Union[str, Callable],
+            gpg_store: GPGStore,
+            passphrase: Optional[str]) -> List[str]:
+    """Decrypt data.
+
+    :param source: data to encrypt.
+    :param output: encrypted data.
+    :param gpg_store: directory containing GnuPG keyrings as gnupg object.
+    :param passphrase: password associated to 'signature_fingerprint'. This
+        parameter value can be set to None if the encrypted file should not be
+        signed.
+    :return: fingerprint or keyid of the signee's key.
+    :raises UserError:
+    """
+    try:
+        if isinstance(output, str):
+            with open(output, "wb") as fout:
+                sender_fprs = gpg_store.decrypt(
+                    source=source,
                     output=cast(io.FileIO, fout),
-                    passphrase=passphrase) as fpr_container:
-                sender_fprs = fpr_container.sender_sig_fingerprints
-            if not os.path.isfile(dest):
-                raise UserError("Output file does not exist")
-        except gpg.cmd.GPGError as e:
-            raise UserError("Failed to decrypt. Error message from gpg:\n" +
-                            format(e)) from e
+                    passphrase=passphrase)
+        else:
+            sender_fprs = gpg_store.decrypt(
+                source=source,
+                output=output,
+                passphrase=passphrase)
+    except gpg.cmd.GPGError as e:
+        raise UserError("Failed to decrypt. Error message from gpg:\n" +
+                        format(e)) from e
     if not sender_fprs:
         warnings.warn("Encrypted package is not signed by the sender")
     elif len(sender_fprs) > 1:
