@@ -23,7 +23,7 @@ class MetaData(_metadata.MetaData):
         try:
             return super().from_dict(d)
         except ValueError as e:
-            raise UserError(format(e))
+            raise UserError(format(e)) from e
 
 
 load_metadata = MetaData.from_dict
@@ -44,4 +44,4 @@ def verify_transfer(metadata: MetaData,
         response = post(portal_endpoint_url, data)
         return json.loads(response)["project_id"]
     except HTTPError as e:
-        raise UserError(json.loads(e.read())[0])
+        raise UserError(json.loads(e.read())[0]) from e
