@@ -1,15 +1,23 @@
 from functools import partial
 from pathlib import Path
-from typing import Callable, Iterable, List, Optional
+from typing import Callable, Optional
 
 from PySide2 import QtCore, QtWidgets
 from PySide2.QtCore import QItemSelectionModel
-from PySide2.QtWidgets import QPushButton
+from PySide2.QtWidgets import QPushButton, QLabel
 
 from .parallel import Worker
 from ..utils.progress import ProgressInterface
 
 
+class MandatoryLabel(QLabel):
+    """A label extension which appends a '*' to label's end to mark the field as required.
+    """
+
+    def __init__(self, label: str):
+        super().__init__(label + " <sup><font color='red'>*</font></sup>")
+
+
 class SelectionButton(QPushButton):
     """A push button extension which connects this button to given selection model.
 
@@ -76,83 +84,6 @@ class ConsoleWidget(QtWidgets.QGroupBox):
         self.textbox.appendPlainText(text)
 
 
-class FileSelectionWidget(QtWidgets.QGroupBox):
-    def __init__(self, title, parent, directory=True, archives_only=False):
-        super().__init__(title, parent)
-        self.path = str(Path.home())
-        self.archives_only = archives_only
-
-        self.file_list_model = QtCore.QStringListModel()
-        self.file_list_view = QtWidgets.QListView(self)
-        self.file_list_view.setModel(self.file_list_model)
-        self.file_list_view.setSelectionMode(
-            QtWidgets.QAbstractItemView.ExtendedSelection)
-        self.file_list_view.setLayout(QtWidgets.QVBoxLayout())
-
-        btn_select_files = QtWidgets.QPushButton('Add files', self)
-        btn_select_files.clicked.connect(self.add_files)
-
-        self.btn_clear_selected = SelectionButton('Remove selected',
-                                                  self.file_list_view.selectionModel())
-        self.btn_clear_selected.clicked.connect(self.clear_selected)
-
-        btn_clear_list = QtWidgets.QPushButton('Clear list', self)
-        btn_clear_list.clicked.connect(self.clear_list)
-
-        buttons = [btn_select_files, self.btn_clear_selected, btn_clear_list]
-
-        if directory:
-            btn_select_dir = QtWidgets.QPushButton('Add directory', self)
-            btn_select_dir.clicked.connect(self.add_directory)
-            buttons.insert(1, btn_select_dir)
-
-        layout = QtWidgets.QGridLayout()
-        layout.addWidget(self.file_list_view, 0, 0, len(buttons), 1)
-        for i, btn in enumerate(buttons):
-            layout.addWidget(btn, i, 1)
-        self.setLayout(layout)
-
-    def _update_paths(self, paths: Iterable[str]) -> None:
-        paths = set(filter(None, paths))
-        if paths:
-            self.path = Path(next(iter(paths))).parent
-        self.file_list_model.setStringList(
-            sorted(set(self.file_list_model.stringList()) | paths))
-        # TODO this shouldn't be necessary, but it's currently used to notify
-        # other (than listview) components about the changes to the files list
-        self.file_list_model.layoutChanged.emit()
-
-    def add_files(self):
-        dialog = QtWidgets.QFileDialog(self)
-        dialog.setFileMode(QtWidgets.QFileDialog.ExistingFiles)
-        dialog.setDirectory(str(self.path))
-        if self.archives_only:
-            dialog.setNameFilters(["Archives *.tar (*.tar)"])
-            dialog.selectNameFilter("Archives *.tar (*.tar)")
-        if dialog.exec_() == QtWidgets.QDialog.Accepted:
-            self._update_paths(dialog.selectedFiles())
-
-    def add_directory(self):
-        directory = (QtWidgets.QFileDialog.getExistingDirectory(
-            self, 'Select directory', str(self.path)))
-        self._update_paths((directory, ))
-
-    def clear_selected(self) -> None:
-        indices = self.file_list_view.selectedIndexes()
-        for index in indices:
-            self.file_list_model.removeRows(index.row(), 1)
-        self.file_list_model.layoutChanged.emit()
-
-    def clear_list(self) -> None:
-        # Clear the selection BEFORE resetting the model
-        self.file_list_view.selectionModel().clearSelection()
-        self.file_list_model.setStringList([])
-        self.file_list_model.layoutChanged.emit()
-
-    def get_list(self) -> List[str]:
-        return self.file_list_model.stringList()
-
-
 class PathInput:
     """Path selection widget with a select button and a show path field."""
 
@@ -172,6 +103,8 @@ class PathInput:
         self.path = path
         self.text.setText("" if path is None else str(path))
         self.text.editingFinished.emit()
+        # 'repaint' invocation is needed on MacOS X
+        self.text.repaint()
 
     def _update_location(self, directory: bool):
         if self.path and self.path.exists():
@@ -199,6 +132,14 @@ class PathInput:
     def btn_label(self, label: str):
         self.btn.setText(label)
 
+    @property
+    def status_tip(self) -> str:
+        return self.text.statusTip()
+
+    @status_tip.setter
+    def status_tip(self, msg: str):
+        self.text.setStatusTip(msg)
+
     def on_path_change(self, fn: Callable[[Optional[Path]], None]) -> None:
         """Run callback when path changes."""
         self.text.editingFinished.connect(lambda: fn(self.path))
@@ -210,15 +151,30 @@ class TabMixin:
 
     def create_progress_bar(self):
         self.progress_bar = QtWidgets.QProgressBar(self)
-        self.progress_bar.hide()
+
+    @staticmethod
+    def _create_disabled_button(action_name: str) -> QPushButton:
+        button = QtWidgets.QPushButton(action_name)
+        button.setEnabled(False)
+        return button
+
+    def set_buttons_enabled(self, enabled: bool):
+        self.btn_run.setEnabled(enabled)
+        self.btn_run.repaint()
+        self.btn_test.setEnabled(enabled)
+        self.btn_test.repaint()
 
     def create_run_panel(self, panel_name: str, action: Callable,
                          action_name: str):
         self.run_panel = QtWidgets.QGroupBox(panel_name)
         layout = QtWidgets.QHBoxLayout()
-        self.btn_test = QtWidgets.QPushButton("Test")
+        self.btn_test = TabMixin._create_disabled_button("Test")
+        # On pressed button, make sure that the focus switches to that button (Mac specific issue)
+        self.btn_test.pressed.connect(self.btn_test.setFocus)
         self.btn_test.clicked.connect(partial(action, dry_run=True))
-        self.btn_run = QtWidgets.QPushButton(action_name)
+        self.btn_run = TabMixin._create_disabled_button(action_name)
+        # On pressed button, make sure that the focus switches to that button (Mac specific issue)
+        self.btn_run.pressed.connect(self.btn_run.setFocus)
         self.btn_run.clicked.connect(action)
         layout.addWidget(self.btn_test)
         layout.addWidget(self.btn_run)
@@ -226,13 +182,10 @@ class TabMixin:
 
     def add_worker_actions(self, worker: Worker):
         """Attach GUI-updating signals to worker"""
-        worker.signals.started.connect(self.progress_bar.show)
         worker.signals.started.connect(
             lambda: self.run_panel.setEnabled(False))
         worker.signals.logging.connect(self.console.write)
         worker.signals.error.connect(lambda e: self.console.append(str(e[1])))
-        worker.signals.finished.connect(lambda: self.progress_bar.setValue(0))
-        worker.signals.finished.connect(self.progress_bar.hide)
         worker.signals.finished.connect(
             lambda: self.run_panel.setEnabled(True))
 
