@@ -2,10 +2,11 @@ from datetime import datetime
 from functools import partial
 
 from PySide2 import QtWidgets, QtCore, QtGui
-from PySide2.QtWidgets import QHeaderView
+from PySide2.QtWidgets import QHeaderView, QLabel
 
-from .component import TabMixin, FileSelectionWidget, PathInput, GuiProgress,\
-    run_dialog, SelectionButton
+from .component import TabMixin, PathInput, GuiProgress, \
+    run_dialog, SelectionButton, MandatoryLabel
+from .file_selection_widget import DirectoryAndFileSelectionWidget
 from .model import TableModel
 from .parallel import Worker
 from ..core.metadata import Purpose
@@ -15,40 +16,38 @@ from ..workflows import encrypt
 class EncryptTab(QtWidgets.QWidget, TabMixin):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        self.setLayout(QtWidgets.QVBoxLayout(self))
+
         self.threadpool = QtCore.QThreadPool.globalInstance()
         self.app_data = self.parent().app_data
 
         files_panel = self.create_files_panel()
-        user_panel = self.create_user_panel()
-        output_panel = self.create_output_panel()
-        self.create_run_panel("Package and Encrypt data", self.encrypt,
+        self.create_run_panel("Package and encrypt data", self.encrypt,
                               "Package && Encrypt")
+        self.app_data.add_listener('encrypt_files', self._enable_buttons)
+        self.app_data.add_listener('encrypt_recipients', self._enable_buttons)
         self.create_console()
         self.create_progress_bar()
 
-        layout = QtWidgets.QVBoxLayout()
-        layout.addWidget(files_panel)
+        self.layout().addWidget(files_panel)
+        self.layout().addLayout(self._create_middle_layout())
+        self.layout().addWidget(self.run_panel)
+        self.layout().addWidget(self.console)
+        self.layout().addWidget(self.progress_bar)
 
-        layout_split = QtWidgets.QHBoxLayout()
-        layout_split.addWidget(user_panel)
-        layout_split.addWidget(output_panel)
-        layout.addLayout(layout_split)
-
-        layout.addWidget(self.run_panel)
-        layout.addWidget(self.console)
-        layout.addWidget(self.progress_bar)
-
-        self.setLayout(layout)
+    def _enable_buttons(self):
+        self.set_buttons_enabled(len(self.app_data.encrypt_recipients) > 0 and
+                                 len(self.app_data.encrypt_files) > 0)
 
     def create_files_panel(self):
-        box = FileSelectionWidget(
-            "Select files and/or directories to encrypt", self)
+        box = DirectoryAndFileSelectionWidget(
+            "Files and/or directories to encrypt", self)
         box.file_list_model.layoutChanged.connect(
             lambda: setattr(self.app_data, "encrypt_files", box.get_list()))
         return box
 
     def create_user_panel(self):
-        box = QtWidgets.QGroupBox("Select data sender and recipients")
+        group_box = QtWidgets.QGroupBox("Keys")
         sender_widget = QtWidgets.QComboBox()
         sender_widget.setModel(self.app_data.priv_keys_model)
         if self.app_data.default_key_index:
@@ -70,6 +69,7 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
             QtWidgets.QTableView.SelectionMode.SingleSelection)
 
         recipients_btn_add = QtWidgets.QPushButton("+")
+        recipients_btn_add.setStatusTip("Add recipient to the list")
         recipients_btn_remove = SelectionButton("Remove selected recipients",
                                                 recipients_output_view.selectionModel())
 
@@ -101,54 +101,42 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         # Set the default value for the sender
         update_sender(sender_widget.currentIndex())
 
-        layout = QtWidgets.QVBoxLayout()
-        layout_form = QtWidgets.QFormLayout()
-        layout_form.addRow("Sender", sender_widget)
-        layout_recipients = QtWidgets.QHBoxLayout()
-        layout_recipients.addWidget(recipients_input_view)
-        layout_recipients.addWidget(recipients_btn_add)
-        layout_form.addRow("Recipients", layout_recipients)
-        layout.addLayout(layout_form)
-        layout.addWidget(recipients_output_view)
-        layout.addWidget(recipients_btn_remove)
+        layout = QtWidgets.QGridLayout(group_box)
+        layout.addWidget(QLabel("Select data sender and add at least one recipient"), 0, 0, 1, 3)
+        layout.addWidget(MandatoryLabel("Sender"), 1, 0, 1, 1)
+        layout.addWidget(sender_widget, 1, 1, 1, 1)
+        layout.addWidget(recipients_input_view, 2, 1, 1, 1)
+        layout.addWidget(recipients_btn_add, 2, 2, 1, 1)
+        layout.addWidget(MandatoryLabel("Recipients"), 3, 0, 1, 1, QtCore.Qt.AlignTop)
+        layout.addWidget(recipients_output_view, 3, 1, 1, 2)
+        layout.addWidget(recipients_btn_remove, 4, 0, 1, 3)
 
-        box.setLayout(layout)
-        return box
+        group_box.setLayout(layout)
+        return group_box
 
-    def create_output_panel(self):
-        box = QtWidgets.QGroupBox("Select output name and location")
+    def _create_middle_layout(self):
+        grid_layout = QtWidgets.QGridLayout()
+        grid_layout.addWidget(self.create_user_panel(), 0, 0, 3, 1)
+        grid_layout.addWidget(self._create_metadata_group_box(), 0, 1, 1, 1)
+        grid_layout.addWidget(self._create_output_group_box(), 1, 1, 1, 1)
+        # Add spacer
+        grid_layout.addWidget(QLabel(), 2, 1, 1, 1)
+        return grid_layout
+
+    def _create_output_group_box(self):
+        group_box = QtWidgets.QGroupBox("Output", self)
+        group_box.setLayout(QtWidgets.QGridLayout(group_box))
         # Create fields
-        transfer_id = QtWidgets.QLineEdit()
-        transfer_id.setStatusTip("(optional) Data Transfer Request ID")
-        transfer_id.setValidator(QtGui.QIntValidator(1, 10**9))
-        purpose = QtWidgets.QComboBox(box)
-        purpose.setStatusTip("Purpose of the package")
-        purpose.addItems(("",) + tuple(Purpose.__members__.keys()))
-        purpose.setCurrentText("")
-
-        offline = QtWidgets.QCheckBox("Offline")
-        offline.setStatusTip("Offline mode: skip DTR ID validation and "
-                             "key refreshing.")
-        offline.setChecked(self.app_data.encrypt_offline)
-        compress = QtWidgets.QCheckBox("Compress inner tarball")
+        compress = QtWidgets.QCheckBox("Compress input data", group_box)
+        compress.setStatusTip("Compress input data files before encryption")
         compress.setChecked(self.app_data.encrypt_compress)
-        output_suffix = QtWidgets.QLineEdit(box)
+        output_suffix = QtWidgets.QLineEdit(group_box)
         output_suffix.setStatusTip("(optional) File name suffix in the "
-	                               "format datetime_suffix.tar")
-        output_suffix.setValidator(
-            QtGui.QRegExpValidator(QtCore.QRegExp(r"[\w-]*")))
+                                   "format datetime_suffix.tar")
+        output_suffix.setValidator(QtGui.QRegExpValidator(QtCore.QRegExp(r"[\w-]*")))
         output_location = PathInput()
+        output_location.status_tip = "Destination folder for the encrypted package"
         # Add actions
-
-        def on_transfer_id_changed(text):
-            self.app_data.encrypt_transfer_id = int(text) if text else None
-        transfer_id.textChanged.connect(on_transfer_id_changed)
-        offline.stateChanged.connect(lambda state:
-            setattr(self.app_data, "encrypt_offline",
-                    state == QtCore.Qt.Checked))
-        purpose.currentTextChanged.connect(lambda text:
-            setattr(self.app_data, "encrypt_purpose",
-                    None if text == "" else Purpose(text)))
         compress.stateChanged.connect(lambda state:
                                       setattr(self.app_data, "encrypt_compress",
                                               state == QtCore.Qt.Checked))
@@ -158,31 +146,51 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         output_location.on_path_change(
             partial(setattr, self.app_data, "encrypt_output_location"))
 
-        layout = QtWidgets.QFormLayout()
-        layout.addRow("DTR ID", transfer_id)
-        layout.addRow("Purpose", purpose)
-        layout.addRow("", offline)
-        layout.addRow("Output suffix", output_suffix)
-        layout.addRow("Output location", output_location.text)
-        layout.addRow("", output_location.btn)
-        layout.addRow("Data package", compress)
-        box.setLayout(layout)
-        return box
+        group_box.layout().addWidget(QLabel("Suffix"), 0, 0, 1, 1)
+        group_box.layout().addWidget(output_suffix, 0, 1, 1, 1)
+        group_box.layout().addWidget(QLabel("Location"), 1, 0, 1, 1)
+        group_box.layout().addWidget(output_location.text, 1, 1, 1, 1)
+        group_box.layout().addWidget(output_location.btn, 2, 1, 1, 1)
+        group_box.layout().addWidget(compress, 3, 0, 1, 2)
+        return group_box
+
+    def _create_metadata_group_box(self):
+        group_box = QtWidgets.QGroupBox("Metadata", self)
+        group_box.setLayout(QtWidgets.QGridLayout(group_box))
+        # Create fields
+        transfer_id = QtWidgets.QLineEdit(group_box)
+        transfer_id.setStatusTip("(optional) Data Transfer Request ID")
+        transfer_id.setValidator(QtGui.QIntValidator(1, 10**9))
+        verify_dtr = QtWidgets.QCheckBox("Verify DTR ID", group_box)
+        verify_dtr.setStatusTip("Verify DTR (Data Transfer Request) ID")
+        verify_dtr.setChecked(not self.app_data.config.offline and self.app_data.encrypt_verify_dtr)
+        if self.app_data.config.offline:
+            verify_dtr.setEnabled(not self.app_data.config.offline)
+            verify_dtr.setStatusTip("Verifying Transfer Request ID in offline mode is not possible")
+        verify_dtr.stateChanged.connect(lambda state:
+            setattr(self.app_data, "encrypt_verify_dtr",
+                    state == QtCore.Qt.Checked))
+        purpose = QtWidgets.QComboBox(group_box)
+        purpose.setStatusTip("(optional) Purpose of the package")
+        purpose.addItems(("",) + tuple(Purpose.__members__.keys()))
+        purpose.setCurrentText("")
+
+        # Add actions
+        def on_transfer_id_changed(text):
+            self.app_data.encrypt_transfer_id = int(text) if text else None
+
+        transfer_id.textChanged.connect(on_transfer_id_changed)
+        purpose.currentTextChanged.connect(lambda text:
+                                           setattr(self.app_data, "encrypt_purpose",
+                                                   None if text == "" else Purpose(text)))
+        group_box.layout().addWidget(QLabel("DTR ID"), 0, 0, 1, 1)
+        group_box.layout().addWidget(transfer_id, 0, 1, 1, 1)
+        group_box.layout().addWidget(verify_dtr, 1, 0, 1, 2)
+        group_box.layout().addWidget(QLabel("Purpose"), 2, 0, 1, 1)
+        group_box.layout().addWidget(purpose, 2, 1, 1, 1)
+        return group_box
 
     def encrypt(self, dry_run=False):
-        warning_msg = []
-        if not self.app_data.encrypt_files:
-            warning_msg.append("Select files for encryption.")
-        if not self.app_data.encrypt_recipients:
-            warning_msg.append("Select at least one recipient.")
-        if warning_msg:
-            msg_warn = QtWidgets.QMessageBox()
-            msg_warn.setWindowTitle("Warning")
-            msg_warn.setText("\n".join(warning_msg))
-            msg_warn.setIcon(QtWidgets.QMessageBox.Warning)
-            msg_warn.exec_()
-            return
-
         progress = GuiProgress()
         progress.updated.connect(self.progress_bar.setValue)
         output_path = str(
@@ -204,11 +212,12 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
             sender=self.app_data.encrypt_sender,
             recipient=self.app_data.encrypt_recipients,
             dtr_id=self.app_data.encrypt_transfer_id,
+            verify_dtr=self.app_data.encrypt_verify_dtr,
             config=self.app_data.config,
             passphrase=pw,
             output_name=output_path,
             dry_run=dry_run,
-            offline=self.app_data.encrypt_offline,
+            offline=self.app_data.config.offline,
             compress=self.app_data.encrypt_compress,
             purpose=self.app_data.encrypt_purpose,
             progress=progress,
