@@ -16,6 +16,7 @@ class LoggerHandler(logging.Handler):
             self.callback(self.format(record))
 
 class WorkerSignals(QObject):
+    started = Signal()
     finished = Signal()
     error = Signal(tuple)
     result = Signal(object)
@@ -52,6 +53,7 @@ class Worker(QRunnable):
             logger_handler.setLevel(logging.INFO)
             for x in self.logger:
                 x.addHandler(logger_handler)
+            self.signals.started.emit()
             result = self.fn(*self.args, **self.kwargs)
         except Exception:  # pylint: disable=broad-except
             exctype, value, _ = sys.exc_info()
