@@ -61,14 +61,14 @@ def upload(files: List[str],  # nosec (False Positive: pkey_password_encoding)
                 try:
                     sftp.posix_rename(remotepath_part, remotepath)
                 except IOError as e:
-                    raise UserError(format(e))
+                    raise UserError(format(e)) from e
             with io.BytesIO(b"") as fl:
                 sftp.putfo(fl=fl,
                            remotepath=str(remote_dir / "done.txt"),
                            callback=progress_callback,
                            confirm=True)
     except ssh_exception.AuthenticationException as e:
-        raise UserError(format(e))
+        raise UserError(format(e)) from e
 
 
 @contextmanager
