@@ -41,9 +41,9 @@ class Config:
     """dataclass holding config data"""
 
     dcc_portal_url: str = "https://portal.dcc.sib.swiss"
-    keyserver_url: str = 'keyserver.dcc.sib.swiss'
+    keyserver_url: Optional[str] = 'keyserver.dcc.sib.swiss'
     gpg_config_dir: str = get_default_gpg_config_dir()
-    key_validation_authority_keyid: str = "881685B5EE0FCBD3"
+    key_validation_authority_keyid: Optional[str] = "881685B5EE0FCBD3"
     sign_encrypted_data: bool = True
     always_trust_recipient_key: bool = True
     repo_url: str = "https://pypi.org"
