@@ -1,14 +1,14 @@
 import platform
-import sys
 from logging import Logger
 from functools import wraps
-from typing import Tuple
+from typing import Tuple, Type, Union
 
 from .. import VERSION_WITH_DEPS
 from ..utils.log import exception_to_message
 
 
-def workflow(logger: Logger, error_list: Tuple[Exception, ...] = ()):
+def workflow(logger: Logger,
+             error_types: Union[Type[BaseException], Tuple[Type[BaseException], ...]] = ()):
     """A function decorator that should be used by every workflow.
 
     At the beggining of the workflow the decorator logs:
@@ -20,10 +20,12 @@ def workflow(logger: Logger, error_list: Tuple[Exception, ...] = ()):
     decorator.
     """
     def dec(f):
-        _f = exception_to_message(error_list=error_list, logger=logger)(f)
+        _f = exception_to_message(error_types=error_types, logger=logger)(f)
+
         @wraps(f)
         def wrapped_f(*args, **kwargs):
-            logger.info(f"{VERSION_WITH_DEPS} (platform: {platform.platform()})")
+            logger.info(
+                f"{VERSION_WITH_DEPS} (platform: {platform.platform()})")
             return _f(*args, **kwargs)
         return wrapped_f
     return dec
