@@ -1,5 +1,7 @@
-import os
+import io
 import json
+import os
+from functools import partial
 from pathlib import Path
 from typing import List, Optional, Callable, Tuple
 
@@ -8,17 +10,16 @@ from ..utils.config import Config
 from ..utils.log import create_logger
 from ..core.archive import write_tar, ArchiveInMemoryFile, ArchiveFile, \
     ArchiveFileBase, METADATA_FILE, METADATA_FILE_SIG, \
-    DATA_ARCHIVE, DATA_FILE_ENCRYPTED, CHECKSUM_FILE, \
+    DATA_FILE_ENCRYPTED, CHECKSUM_FILE, \
     CONTENT_FOLDER
 from ..core.filesystem import search_files_recursively, \
     check_file_read_permission, form_output_tar_name, delete_files
-from ..core.crypt import retrieve_keys, encrypt_and_sign_file, \
+from ..core.crypt import retrieve_keys, encrypt_and_sign, \
     detach_sign_file, check_password
-from ..core.checksum import compute_checksum_sha256, write_checksums
+from ..core.checksum import write_checksums, compute_checksum_on_write
 from ..core.error import UserError
 from ..core.metadata import MetaData, alnum_str, Purpose, HexStr1024, HexStr256, verify_transfer
-from ..utils.progress import ProgressInterface, subprogress, \
-    progress_file_iter, with_progress
+from ..utils.progress import ProgressInterface, subprogress, progress_file_iter
 
 DATE_FMT_FILENAME = "%Y%m%dT%H%M%S"
 logger = create_logger(__name__)
@@ -76,6 +77,7 @@ def encrypt(files: List[str],
             output_name: Optional[str] = None,
             dry_run: bool = False,
             offline: bool = False,
+            verify_dtr: bool = True,
             compress: bool = True,
             purpose: Optional[Purpose] = None,
             progress: ProgressInterface = None) -> None:
@@ -97,9 +99,8 @@ def encrypt(files: List[str],
         # Retrieve the lowest common directory of all input files/directories.
         root_dir = os.path.commonpath([Path(x).absolute().parent.as_posix()
                                        for x in files])
-        if dtr_id is None and not offline:
-            raise UserError("DTR (Data Transfer Request) ID is missing. Use "
-                            "offline mode if you don't require DTR ID.")
+        if dtr_id is None and verify_dtr:
+            raise UserError("DTR (Data Transfer Request) ID is missing.")
         if output_name:
             check_arg_value(Path(output_name).name,
                             "output name", output_name_str)
@@ -134,7 +135,7 @@ def encrypt(files: List[str],
         if dtr_id is not None:
             if purpose is None:
                 raise UserError("DTR ID specified but `purpose` is missing")
-            if not offline:
+            if not offline and verify_dtr:
                 try:
                     project_id = verify_transfer(
                         metadata=MetaData(
@@ -176,35 +177,30 @@ def encrypt(files: List[str],
                                        mode='rb',
                                        progress=scaled_progress)))
 
-    with logger.log_task("Compress input data [this may take a while]"):
-        # Create a new tar.gz file and add all input files to it.
-        tarball_file = os.path.join(config.temp_dir, DATA_ARCHIVE)
-        archives: Tuple[ArchiveFileBase, ...] = \
-            ((ArchiveInMemoryFile(CHECKSUM_FILE, checksums),) +
-             tuple(ArchiveFile(a_path, f) for a_path, f in zip(
-                archive_paths, progress_file_iter(files=files_to_encrypt,
-                                                  mode='rb',
-                                                  progress=scaled_progress)))
-            )
-        with subprogress(progress,
-                         step_completion_increase=0.6) as scaled_progress:
-            write_tar(archives,
-                      output_file=tarball_file,
-                      mode="w:gz" if compress else "w")
-
-    with logger.log_task("Encrypt input data"):
+    with logger.log_task("Compress and encrypt input data [this may take a while]"):
         # Encryption is done with the recipient's public key and the optional
         # signing with the user's (i.e sender) private key. The user's private
         # key passphrase is needed to sign the encrypted file.
         encrypted_file = os.path.join(config.temp_dir, DATA_FILE_ENCRYPTED)
-        try:
-            with subprogress(progress,
-                             step_completion_increase=0.1) as scaled_progress:
-                tarball_file_object = with_progress(f=open(tarball_file, 'rb'),
-                                                    progress=scaled_progress)
-                encrypt_and_sign_file(
-                    file_object_to_encrypt=tarball_file_object,
-                    output_file=encrypted_file,
+        encrypted_checksum_buf = io.StringIO()
+        with subprogress(progress,
+                         step_completion_increase=0.75) as scaled_progress:
+            # Create a tar archive contaiing all input files
+            archive_content: Tuple[ArchiveFileBase, ...] = \
+                ((ArchiveInMemoryFile(CHECKSUM_FILE, checksums),) +
+                tuple(ArchiveFile(a_path, f) for a_path, f in zip(
+                    archive_paths, progress_file_iter(files=files_to_encrypt,
+                                                      mode='rb',
+                                                      progress=scaled_progress)))
+                )
+            with open(encrypted_file, "wb") as fout:
+                encrypt_and_sign(
+                    source=partial(write_tar,
+                                   archive_content,
+                                   mode="w|gz" if compress else "w|"),
+                    output=partial(compute_checksum_on_write,
+                                   fout=fout,
+                                   checksum_buffer=encrypted_checksum_buf),
                     gpg_store=config.gpg_store,
                     recipients_fingerprint=[key.fingerprint
                                             for key in recipients_pub_key],
@@ -213,21 +209,9 @@ def encrypt(files: List[str],
                     passphrase=enforce_passphrase(passphrase) if
                     config.sign_encrypted_data else None,
                     always_trust=config.always_trust_recipient_key)
-        finally:
-            # Delete the non-encrypted tarball file. It's no longer needed at
-            # this point.
-            delete_files(tarball_file)
+        encrypted_checksum = encrypted_checksum_buf.read()
 
     try:
-        with logger.log_task("Compute checksum of encrypted file"):
-            # Compute sha256 sum on encrypted file, so intermediate integrity
-            # checks can be carried-out without having to decrypt the data.
-            with subprogress(progress,
-                             step_completion_increase=0.05) as scaled_progress:
-                with open(encrypted_file, 'rb') as f:
-                    encrypted_checksum = compute_checksum_sha256(
-                        with_progress(f=f, progress=scaled_progress))
-
         with logger.log_task("Generate metadata file"):
             # Create a dictionary with all the info we want to store in the
             # .json file, then pass this dictionary to json.dump that will
@@ -260,7 +244,7 @@ def encrypt(files: List[str],
                                (METADATA_FILE_SIG, metadata_signature_bytes))
             with subprogress(progress,
                              step_completion_increase=0.05) as scaled_progress:
-                archives = (
+                archive_content = (
                     tuple(ArchiveInMemoryFile(path, content)
                           for path, content in in_memory_files) +
                     tuple(ArchiveFile(os.path.relpath(f.name, start=config.temp_dir), f)
@@ -269,7 +253,7 @@ def encrypt(files: List[str],
                             mode='rb',
                             progress=scaled_progress))
                     )
-                write_tar(archives, output_file=output_name, mode='w')
+                write_tar(archive_content, output=output_name, mode='w')
     finally:
         delete_files(encrypted_file)
 
